@@ -1,6 +1,6 @@
 use crate::auditwheel::PlatformTag;
 use crate::build_context::{BridgeModel, ProjectLayout};
-use crate::cross_compile::{find_sysconfigdata, is_cross_compiling, parse_sysconfigdata};
+use crate::cross_compile::{find_sysconfigdata, parse_sysconfigdata};
 use crate::python_interpreter::InterpreterKind;
 use crate::BuildContext;
 use crate::CargoToml;
@@ -356,11 +356,11 @@ fn has_abi3(cargo_metadata: &Metadata) -> Result<Option<(u8, u8)>> {
             let min_abi3_version = pyo3_crate
                 .features
                 .iter()
-                .filter(|x| x.starts_with("abi3-py") && x.len() == "abi3-pyxx".len())
+                .filter(|x| x.starts_with("abi3-py") && x.len() >= "abi3-pyxx".len())
                 .map(|x| {
                     Ok((
                         (x.as_bytes()[7] as char).to_string().parse::<u8>()?,
-                        (x.as_bytes()[8] as char).to_string().parse::<u8>()?,
+                        x[8..].parse::<u8>()?,
                     ))
                 })
                 .collect::<Result<Vec<(u8, u8)>>>()
@@ -515,7 +515,7 @@ pub fn find_interpreter(
                 }
             }
 
-            if binding_name == "pyo3" && target.is_unix() && is_cross_compiling(target)? {
+            if binding_name == "pyo3" && target.is_unix() && target.cross_compiling() {
                 if let Some(cross_lib_dir) = std::env::var_os("PYO3_CROSS_LIB_DIR") {
                     println!("⚠️ Cross-compiling is poorly supported");
                     let host_python = &interpreter[0];
